# bucket_dumper.py:  A small bucket brigade dumper. 
#
# Copyright (C) 2000, Jon Travis (jtravis@p00p.org)
#
# Description:
#   This module is primarily for module authors who wish to debug their
#   handling of bucket brigades.  This module will dump all brigade
#   information it receives to stderr.  This is often Apache's error_log
#
# How to use it:
#   Simply add the following line to your httpd.conf configuration file:
#     SnakeModule bucket_dumper.BucketDumper
#   This assumes that SnakeModuleDir has been setup correctly to contain
#   the path that this module resides in.
#

import mod_snake
import sys
try:    from cStringIO import StringIO
except: from StringIO import StringIO

FILTERNAME        = 'bucketdumper'

class BucketDumper:
    def __init__(self, module):
        module.add_hook('insert_filter',     self.insert_filter)
        module.register_output_filter(FILTERNAME, self.bucketdumper_filter,
                                      mod_snake.AP_FTYPE_CONNECTION)

    def insert_filter(self, per_dir, per_svr, r):
        r.add_output_filter(FILTERNAME, None)
        return mod_snake.OK
    
    def bucketdumper_filter(self, per_dir, per_svr, filter, brigade):
        node = brigade.get_head()
        while node:
            if node.is_eos:
                sys.stderr.write('Bucket: %4d (EOS)\n' % (node.length,))
            else:
                sys.stderr.write('Bucket: %4d ("%s")\n' % (node.length,
                                                           `node.read(1)[0:10]`
                                                           ))

            node = brigade.get_next(node)
            
        sys.stderr.flush()
        filter.next.pass_brigade(brigade)
        return mod_snake.OK
