"""
01_hello_world.py: An extremely simple hello world sample.  This module
                   sends the string 'Hello World' to the client when
                   the URL /hello_world is requested.


*CONCEPTS:
- Content generation via the content_handler

*TO USE:
Insert the following lines in your httpd.conf file:
----
SnakeModuleDir /path/to/mod_snake/examples/tut
SnakeModule 01_hello_world.Hello_World
----
where /path/to/mod_snake is the path to the extracted mod_snake source.

Start the webserver, and get: http://localhost/hello_world
to see the output.
"""

import mod_snake

class Hello_World:
    def __init__(self, module):
        # The module.add_hook method tells Apache to call our own content
        # handler when Apache needs to generate content.
        module.add_hook('content_handler',   self.content_handler)
        
    def content_handler(self, per_dir, per_svr, r):
        # Ensure that the URI the client is requesting is '/hello_world' and
        # that they are asking for the real thing, not just the HEAD
        if r.uri == '/hello_world' and r.method == 'GET':
            # First set the type of data we are returning to the browser
            r.content_type = 'text/html'
            # Secondly send all the outgoing headers
            r.send_http_header()
            # Finally send the message 'Hello World' to the browser
            r.rwrite('Hello world!\n')

            # Lastly, return mod_snake.OK to tell Apache that we have
            # processed the request, and that no more content_handlers from
            # other modules should be run
            return mod_snake.OK

        # If we didn't process the data that Apache requested, then we
        # return mod_snake.DECLINED to signify that the content_handler in
        # other modules should be run
        return mod_snake.DECLINED

        
