/* ====================================================================
 * Copyright (c) 2001 Jon Travis.  All rights reserved.
 * See the file COPYING for information on usage and redistribution.
 * ====================================================================
 */

#ifndef MOD_SNAKE_DOT_H
#define MOD_SNAKE_DOT_H

#include "httpd.h"
#define CORE_PRIVATE
#include "http_config.h"
#undef CORE_PRIVATE
#include "http_core.h"
#include "http_log.h"
 
#ifndef APACHE13
#include "http_connection.h"
#include "snake_apr_compat.h"
#include "util_filter.h"
#endif

#include "http_protocol.h"
#include "http_request.h"
#include "Python.h"
#include "config.h"
#ifndef APACHE13
#include "apr_network_io.h"
#include "apr_strings.h"
#endif
#include "snake_precompile.h"
#include <sys/param.h>
#include "mod_snake_buf.h"

#ifdef APACHE13
#define APR_SUCCESS 0
#define ap_pool_t pool
#define ap_context_t pool
#define ap_array_header_t array_header
#define ap_table_t table
#define ap_table_entry_t table_entry
#define ap_stat stat
#define ap_size_t size_t
#define ap_ssize_t ssize_t
#define ap_time_t time_t
#define ap_status_t int

/* Apache 2.0 entities which simply don't exist in any fashion under 1.3
   are just handled as void *'s, and exceptions are raised when modules 
   try to use them */
typedef void *apr_bucket_brigade;
typedef void *ap_filter_t;
typedef void *apr_bucket;
#define AP_FTYPE_CONTENT     -1
#define AP_FTYPE_CONNECTION  -1
#define AP_FTYPE_HTTP_HEADER -1
#define AP_FTYPE_TRANSCODE   -1
#define AP_FTYPE_NETWORK     -1
#endif

#define SNAKE13_UNSUP(resval) \
  do { \
    PyErr_SetString(PyExc_SystemError, "operation unsupported in Apache 1.3");\
    return (resval); \
  }  while(0)
#define SNAKE20_UNSUP(resval) \
  do { \
    PyErr_SetString(PyExc_SystemError, "operation unsupported in Apache 2.0");\
    return (resval); \
  }  while(0)
#define SNAKE13_UNSUP_VOID() \
  do { \
    PyErr_SetString(PyExc_SystemError, "operation unsupported in Apache 1.3");\
    return; \
  } while(0)
#define SNAKE20_UNSUP_VOID() \
  do { \
    PyErr_SetString(PyExc_SystemError, "operation unsupported in Apache 2.0");\
    return; \
  } while(0)

typedef struct mod_snake_server_cfg_st {
  ap_pool_t         *cfgpool;        /* The pool the cfg was created with   */
  ap_array_header_t *svr_config_data;/* Array of ModSnakeCallbackData --
					per-server configuration data       */
  server_rec        *server;         /* Server which created us             */
  struct mod_snake_server_cfg_st *parent;  /* If server configurations are
					      merged, this points to the base
					      svr-cfg that is generated from
					      the (base, new) pair */
} ModSnakeSCfg;

typedef struct mod_snake_dir_cfg_st {
  ap_pool_t         *cfgpool;
  ap_array_header_t *dir_config_data; /* Array of ModSnakeCallbackData with
                                         all the callback datas for the 
                                         modules within this dir            */
  char              *path;
  struct mod_snake_dir_cfg_st  *parent; /* If directory configurations are 
                                         merged, this points to the base
                                         dir-cfg that is generated from the
                                         (base, new) pair */
} ModSnakeDCfg;


#define SNAKE_HOOK_DIRECTIVE                -3  /* Directive callbacks       */
#define SNAKE_HOOK_INIT                     -2  /* Initialization - fake hook*/
#define SNAKE_HOOK_NO_HOOK                  -1  /* Not a hook.. ;-)          */
#define SNAKE_HOOK_CREATE_DIR_CONFIG         0
#define SNAKE_HOOK_MERGE_DIR_CONFIG          1
#define SNAKE_HOOK_CREATE_SVR_CONFIG         2
#define SNAKE_HOOK_MERGE_SVR_CONFIG          3
#define SNAKE_HOOK_POST_CONFIG               4
#define SNAKE_HOOK_OPEN_LOGS                 5
#define SNAKE_HOOK_HEADER_PARSER             6
#define SNAKE_HOOK_POST_READ_REQUEST         7
#define SNAKE_HOOK_LOG_TRANSACTION           8
#define SNAKE_HOOK_TRANSLATE_NAME            9
#define SNAKE_HOOK_CHECK_USER_ID             10
#define SNAKE_HOOK_FIXUPS                    11
#define SNAKE_HOOK_TYPE_CHECKER              12
#define SNAKE_HOOK_ACCESS_CHECKER            13
#define SNAKE_HOOK_AUTH_CHECKER              14
#define SNAKE_HOOK_CONTENT_HANDLER           15
#define SNAKE_HOOK_PRE_CONNECTION            16
#define SNAKE_HOOK_PROCESS_CONNECTION        17
#define SNAKE_HOOK_INSERT_FILTER             18
#define SNAKE_HOOK_RUN_FILTER                19
#ifdef APACHE13
#define SNAKE_HOOK_NUM_HOOKS                 16  /* Update when above changes*/
#else
#define SNAKE_HOOK_NUM_HOOKS                 20  /* Update when above changes*/
#endif

#define SNAKE_HOOK_BOFFO       -1     /* Means that the hook really doesn't
                                         have an order type */
#define SNAKE_HOOK_FIRST        0     
#define SNAKE_HOOK_ALL          1
#define SNAKE_HOOK_VOID         2

#define SNAKE_RWRITE_BUFFER_SIZE  (1024 * 8)

#define SNAKE_FILTERS_PREFIX      "MSF:"
#define SNAKE_FILTERS_NAME_MAX    80
#define SNAKE_FILTERS_MAX         40

#define SNAKE_FILTERS_INREQUEST   0   /* request_rec input filter  */
#define SNAKE_FILTERS_OUTREQUEST  1   /* request_rec output filter */
#define SNAKE_FILTERS_INCONN      2   /* conn_rec input filter     */
#define SNAKE_FILTERS_OUTCONN     3   /* conn_rec output filter    */
#define SNAKE_FILTERS_NTYPES      4

typedef struct {
  char name[SNAKE_FILTERS_NAME_MAX];
  PyObject *func;
} ModSnakeRegFilter;

typedef struct mod_snake_pymod_st {
  ap_pool_t *ap_p;                 /* Pool the pymod was created with       */
  char *name;                      /* As per the config directive           */

  /* The below PyObjects are only referenced at most 1 time from the
     main program.  */
  PyObject *module;                /* Loaded python module (from import)    */
  PyObject *mod_obj;               /* Instantiation of the object           */
  PyInterpreterState *interp;      /* Interpreter the threads run in for
                                      this module                           */
  PyThreadState  *main_state;      /* State in which the interp was created */
  struct {
    int profile;                   /* 1 if this module should be profiled
                                      via profile.py, else 0                */
    PyObject *profile_obj;         /* The profile.Profile() object we are 
                                      using to profile calls to this module */
    PyObject *profile_func;        /* Function object to call to profile.
                                      Should be profile_obj.runcall         */
  } profile_info;

  int valid;                       /* Whether or not the module is actually
                                      loaded.  If the module IS loaded
                                      then the above PyObjects will be 
                                      non-NULL                              */
  /* Hooks */
  int current_hook;                /* Designates which hook is currently
                                      being called out to.  -1 indicates
                                      no hook being called                  */
  PyObject *hooks[SNAKE_HOOK_NUM_HOOKS];
  ModSnakeRegFilter filters[SNAKE_FILTERS_NTYPES][SNAKE_FILTERS_MAX];
} ModSnakePyMod;

/*
 * ModSnakeIntFilter:  Internal function object which gets passed around in
 *                     filter context information.
 */

typedef struct mod_snake_internal_filter_st {
  ModSnakePyMod *pymod;
  int filter_idx;        /* Index into pymod->filter */
  PyObject *data;        /* Callback data            */
} ModSnakeIntFilter;

typedef struct mod_snake_callback_data_st {
  PyObject *data;          /* Data the module expects as a callback */
  ModSnakePyMod *module;   /* Module associated with this data      */
} ModSnakeCallbackData;


typedef struct mod_snake_internal_directive_st {
  PyObject *funcptr;       /* Actual object to call                  */
  ModSnakePyMod *pymod;    /* Module owning this directive           */
  int valid;
} ModSnakeIntDirective;

/* Since we still want to have per-connection and per-request 
   configuration objects, we have to be a bit clever.  Apache 1.3 
   doesn't have per-conn configuration objects, however Apache 2.0
   does.  In the case of 1.3, we simply want to map one of the types
   onto the other */

#define SNAKE_CONNCONFIG_COMMON \
  ModSnakePyMod *pymod;  /* Currently executing PyMod */ \

/* ModSnakeReqConfig: per-request configuration object  */
typedef struct mod_snake_request_config_st {
  SNAKE_CONNCONFIG_COMMON
  PyObject    *notes;
#ifndef APACHE13
  ModSnakeBuf *curbuf;
  int          bufused;
#endif
} ModSnakeReqCfg;

#ifdef APACHE13
typedef struct mod_snake_request_config_st ModSnakeConnCfg;
#else
/* ModSnakeConnConfig: per-connection configuration object */
typedef struct mod_snake_conn_config_st {
  SNAKE_CONNCONFIG_COMMON
} ModSnakeConnCfg;
#endif

#ifdef APACHE13
#define AP_ARRAY_POOL(p) (p)->pool
#else
#define AP_ARRAY_POOL(p) (p)->cont
#endif

#ifdef APACHE13
#define AP_CLEANUP_RESTYPE void
#define AP_CLEANUP_RESULT(x) 
#else
#define AP_CLEANUP_RESTYPE ap_status_t
#define AP_CLEANUP_RESULT(x) (x)
#endif

/* Swig-made modules */
extern void initmod_snakec(void);
extern void initaprc(void);
extern PyObject *mod_snake_get_generic_shadowclass(void *obj, 
                                                   PyObject *(*func)(void *),
                                                   char *sclass);
extern PyObject *mod_snake_get_ap_table_t_shadowclass(ap_table_t *table);
extern PyObject *mod_snake_get_conn_rec_shadowclass(conn_rec *c);
extern PyObject *mod_snake_get_server_rec_shadowclass(server_rec *s);
extern PyObject *mod_snake_get_server_addr_rec_shadowclass(server_addr_rec *s);
extern PyObject *mod_snake_get_request_rec_shadowclass(request_rec *rec);
extern PyObject *mod_snake_get_modsnakepymod_shadowclass(ModSnakePyMod *pymod);
extern PyObject *mod_snake_get_require_line_shadowclass(require_line *reqline);
extern PyObject *mod_snake_get_apr_bucket_brigade_shadowclass(apr_bucket_brigade *bb);
extern PyObject *mod_snake_get_ap_filter_t_shadowclass(ap_filter_t *filter);

ModSnakePyMod *mod_snake_find_module_byname(ap_array_header_t *mods, 
                                            const char *name);
extern int mod_snake_add_directive(ModSnakePyMod *pymod,
                                   const char *directive_name, int location, 
                                   int functype, PyObject *funcptr, 
                                   const char *func_desc);

extern int mod_snake_initmod_internal(char *modname, SnakePrecompiledPy *);
extern int mod_snake_setup_syspath(ap_array_header_t *moddirs);
extern void mod_snake_unload_module(ModSnakePyMod *pymod);
extern int mod_snake_import_module(ModSnakePyMod *pymod, server_rec *s);
extern int mod_snake_load_module(ModSnakePyMod *pymod, 
                                 ap_array_header_t *syspath, server_rec *s);

extern PyThreadState *mod_snake_create_thread_and_lock(PyInterpreterState *);
extern void mod_snake_destroy_thread_and_unlock(PyThreadState *tstate);

extern int mod_snake_run_filter_hook(ModSnakePyMod *pymod,
				     PyObject *func,
				     PyObject *per_dir_data,
				     PyObject *per_svr_data,
				     ap_filter_t *f, 
				     apr_bucket_brigade *brigade, int do_hook);
extern PyObject * 
mod_snake_run_hook_merge_dir_config(ModSnakePyMod *pymod,
                                    PyObject *base_cdata, PyObject *new_cdata,
                                    PyObject *callback);

extern PyObject * 
mod_snake_run_hook_merge_svr_config(ModSnakePyMod *pymod,
				    PyObject *base, PyObject *newc,
				    PyObject *callback);

extern PyObject * 
mod_snake_run_hook_create_svr_config(ModSnakePyMod *pymod, server_rec *s,
                                     ModSnakeSCfg *svr_cfg,
                                     PyObject *callback);
extern PyObject * 
mod_snake_run_hook_create_dir_config(ModSnakePyMod *pymod,
                                     ModSnakeDCfg *dir_cfg,
                                     PyObject *callback);

extern 
int mod_snake_run_generic_request_rec(ModSnakePyMod *pymod, request_rec *r,
				      PyObject *per_dir_data,
				      PyObject *per_svr_data, 
                                      PyObject *callback, int cb_num);
extern 
int mod_snake_run_generic_conn_rec(ModSnakePyMod *pymod, conn_rec *c,
				   PyObject *per_svr_data,
                                   PyObject *callback, int cb_num);

extern int mod_snake_hooknum_to_type(int hookno);
extern int mod_snake_hookname_to_num(const char *hookname);
extern const char *mod_snake_hooknum_to_name(int hookno);
extern void mod_snake_run_hook_post_config(ModSnakePyMod *pymod, 
					   ModSnakeSCfg *scfg);
extern void mod_snake_run_hook_open_logs(ModSnakePyMod *pymod, 
					 ModSnakeSCfg *scfg);

extern void mod_snake_log_error_unformat(const char *file, int line, int level,
				       ap_status_t status, const server_rec *s,
					 const char *logstr);

extern void mod_snake_log_error(const char *file, int line, int level,
                ap_status_t status, const server_rec *s, const char *fmt, ...)
#ifdef __GNUC__
                __attribute__((format(printf,6,7)))
#endif
     ;

extern 
PyObject *mod_snake_pyeval_callobject(ModSnakePyMod *pymod, PyObject *func, 
                                      PyObject *args, int hook_num);
PyObject *
mod_snake_find_svr_cback_data(ModSnakePyMod *pymod, ModSnakeSCfg *scfg);
PyObject *
mod_snake_find_dir_cback_data(ModSnakePyMod *pymod, ModSnakeDCfg *dcfg);
PyObject *
mod_snake_find_cback_data(ModSnakePyMod *pymod, ap_array_header_t *cfgs);


extern ModSnakeSCfg *mod_snake_get_svr_cfg(server_rec *svr);
extern ModSnakeDCfg *mod_snake_get_dir_cfg(request_rec *req);
extern ModSnakeConnCfg *mod_snake_get_conn_cfg(conn_rec *conn, request_rec *r);
extern ModSnakeReqCfg *mod_snake_get_req_cfg(request_rec *req);
extern ap_array_header_t *mod_snake_get_modules(void);
extern server_rec *mod_snake_get_main_server(void);
extern int mod_snake_is_locked(void);
extern void mod_snake_set_curpymod(conn_rec *, request_rec *, ModSnakePyMod *);
extern ModSnakePyMod *mod_snake_get_curpymod(conn_rec *, request_rec *);
extern PyThreadState *mod_snake_main_threadstate(void);

#if 0
#define MS_TRACE fprintf(stderr, "%-40s %-15s %d\n", __PRETTY_FUNCTION__, \
                         __FILE__, __LINE__)
#else
#define MS_TRACE
#endif

#endif
