/* ====================================================================
 * Copyright (c) 2001 Jon Travis.  All rights reserved.
 * See the file COPYING for information on usage and redistribution.
 * ====================================================================
 */

#ifndef APACHE13

#include "mod_snake.h"
#include "mod_snake_bucket.h"

/*
 * ModSnakePyBucket:  This bucket type gives the program a big speedup in
 *                    that it can use the memory directly from what is in
 *                    the Python program, and refer to specific chunks of it.
 *                    Since Python strings are immutable, this is handy!
 *
 *                    On creation of the pybucket, the string object passed in
 *                    is INCREF'd one time, to signify that it is now being
 *                    used.  When the bucket refcount goes to 0, then string
 *                    object is DECREF'd, and the PyBucket is deleted.  
 *
 *                    On splitting of the bucket, the newly created bucket will
 *                    INCREF the string object 
 *
 *                    The reason that Python refcounting is used, as well as
 *                    bucket refcounting, is that there must be some way to
 *                    gauge when to destroy the bucket's malloc()'ed data.
 */

typedef struct {
  apr_bucket_refcount  refcount; 
  PyObject            *py_str;
  int                  offset;
  int                  len;
} ModSnakePyBucket;

static 
apr_status_t mod_snake_pybucket_read(apr_bucket *bucket, const char **str,
				     apr_size_t *len, apr_read_type_e block)
{
  apr_bucket_shared *sharedbucket = bucket->data;
  ModSnakePyBucket *pybucket = sharedbucket->data;
  const char *py_data = PyString_AS_STRING(pybucket->py_str);

  *str = py_data + sharedbucket->start;
  *len = sharedbucket->end - sharedbucket->start;
  return APR_SUCCESS;
}

static
void mod_snake_pybucket_destroy(void *data){
  ModSnakePyBucket *pybucket;

  /* Check to see if the reference count is up */
  if((pybucket = apr_bucket_destroy_shared(data)) == NULL)
    return;

  Py_DECREF(pybucket->py_str);
  free(pybucket);
}

static const apr_bucket_type_t mod_snake_bucket_type = {
  "ModSnake Bucket", 4,
  mod_snake_pybucket_destroy,
  mod_snake_pybucket_read,
  apr_bucket_setaside_notimpl,
  apr_bucket_split_shared,
};

static apr_bucket *mod_snake_pybucket_make(apr_bucket *bucket,PyObject *strobj,
					   int offset, int len)
{
  ModSnakePyBucket *pb_data;

  if((pb_data = malloc(sizeof(*pb_data))) == NULL)
    return NULL;

  pb_data->py_str  = strobj;
  pb_data->offset  = offset;
  pb_data->len     = len;

  if(!(bucket = apr_bucket_make_shared(bucket, pb_data, offset, offset +len))){
    free(pb_data);
    return NULL;
  }

  Py_INCREF(strobj);
  bucket->type     = &mod_snake_bucket_type;
  return bucket;
}

apr_bucket *mod_snake_pybucket_create(PyObject *strobj, int offset, int len){
  /* This lame little beasty is a macro, unfortunately.  It performs
     the return, creation of vars, and other lame stuff. */
  apr_bucket_do_create(mod_snake_pybucket_make(b, strobj, offset, len));
}



#endif
