/* ====================================================================
 * Copyright (c) 2001 Jon Travis.  All rights reserved.
 * See the file COPYING for information on usage and redistribution.
 * ====================================================================
 */

#include "mod_snake.h"

/* Include internal Python stuffs */
#include "grammar.h"
#include "node.h"
#include "parsetok.h"
#include "errcode.h"
#include "compile.h"
#include "eval.h"
#include "marshal.h"
#include "snake_precompile.h"

/*
 * mod_snake_exec_precompile:  Execute a precompiled python script in the
 *                             current interpreter/thread.  
 *
 * Arguments:                  modname = Name to import the module as.
 *                             precomp = Precompiled object to execute.
 *
 * Return values:              Returns an INCREF'd PyObject which is the
 *                             module loaded.
 */

PyObject *mod_snake_exec_precompile(char *modname,SnakePrecompiledPy *precomp){
  PyObject *pcode, *pmod;

  pcode = PyMarshal_ReadObjectFromString(precomp->compdata, precomp->size);
  if(pcode == NULL)
    return NULL;

  pmod = PyImport_ExecCodeModule(modname, pcode);
  Py_XDECREF(pcode);
  return pmod;  /* Incref'd */
}
